#pragma once
#include "UIAbstract.h"
#include "definitions.h"
#include "sound_functions.h"
#include "LCD_Manager.h"

UIAbstract *executeAction(int actionId);

namespace NSLockpicking
{
    const byte positionMax = 2 * CHR_PER_ROW + 2 * (ROWS - EXCLUDED_ROWS - 1) - 1;

    byte indexFromPosition(byte position)
    {
        const byte lastRow = (ROWS - EXCLUDED_ROWS) * CHR_PER_ROW;
        byte startValue = 0, endValue = CHR_PER_ROW / 2;
        if (position < endValue)
        {
            return lastRow + endValue - 1 - position;
        }
        startValue = endValue;
        endValue += ROWS - EXCLUDED_ROWS;
        if (position < endValue)
        {
            return (endValue - 1 - position) * CHR_PER_ROW;
        }
        startValue = endValue;
        endValue += CHR_PER_ROW - 1;
        if (position < endValue)
        {
            return position - startValue + 1;
        }
        startValue = endValue;
        endValue += ROWS - EXCLUDED_ROWS;
        if (position < endValue)
        {
            return (position - startValue + 2) * CHR_PER_ROW - 1;
        }
        endValue += CHR_PER_ROW / 2;
        return lastRow + CHR_PER_ROW / 2 + endValue - 2 - position;
    }

    void initLockDisplay(char *lockDisplay)
    {
        char *row = lockDisplay + (ROWS - EXCLUDED_ROWS) * CHR_PER_ROW;
        for (int i = 0; i < CHR_PER_ROW; i++)
        {
            lockDisplay[i] = SYMBOL_EDGE;
            row[i] = SYMBOL_EDGE;
        }
        const char remainingRowCount = ROWS - EXCLUDED_ROWS - 1;
        row = lockDisplay;
        for (int i = 0; i < remainingRowCount; i++)
        {
            row += CHR_PER_ROW;
            row[0] = SYMBOL_EDGE;
            for (int j = 1; j < CHR_PER_ROW - 1; j++)
            {
                row[j] = ' ';
            }
            row[CHR_PER_ROW - 1] = SYMBOL_EDGE;
        }
    }

    class UI : public UIAbstract
    {
    private:
        byte positionCorrect, positionSelected;

    public:
        UI()
        {
            positionCorrect = random(0, positionMax);
        }

        void OnUpdate()
        {
            clearDisplay(WHITE);
            char buffer[CHR_PER_ROW + 1];
            positionSelected = positionMax - round(((float)analogRead(Potentiometer_pin)) / 1024 * positionMax);
            if (cheatLockVision)
            {
                sprintf(buffer, "Lockpicking %d", positionSelected);
                setStr(buffer, 0, 0, WHITE);
            }
            else
                setStr(" Lockpicking  ", 0, 0, WHITE);
            const byte characterCount = 4 * CHR_PER_ROW;
            char locks[characterCount + 1];
            locks[characterCount] = 0;
            initLockDisplay(locks);
            const byte index = indexFromPosition(positionSelected);
            locks[index] = SYMBOL_SELECTED;
            setStr(locks, 0, CHR_HEIGHT, BLACK);
            sprintf(buffer, "Picks: %d", player.lockpicks);
            setStr(buffer, 3 * CHR_WIDTH, 3 * CHR_HEIGHT, BLACK);
            if (cheatLockVision)
            {
                sprintf(buffer, "Cheat: %d", positionCorrect);
                setStr(buffer, 3 * CHR_WIDTH, 4 * CHR_HEIGHT, BLACK);
            }
            updateDisplay();
        }

        UIAbstract *OnButtonPress(byte index)
        {
            if (!player.lockpicks)
                return nullptr;
            if (positionSelected == positionCorrect)
            {
                soundSuccess();
                return executeAction(ACTION_TREASAURE);
            }
            else
            {
                soundRange(abs(positionSelected - positionCorrect), 0, positionMax);
                --player.lockpicks;
            }
            Serial.print("AAA\n");
            return nullptr;
        }
    };
}