/* Nokia 5100 LCD Example Code
   Graphics driver and PCD8544 interface code for SparkFun's
   84x48 Graphic LCD.
   https://www.sparkfun.com/products/10168

  by: Jim Lindblom
    adapted from code by Nathan Seidle and mish-mashed with
    code from the ColorLCDShield.
  date: October 10, 2013
  license: Officially, the MIT License. Review the included License.md file
  Unofficially, Beerware. Feel free to use, reuse, and modify this
  code as you see fit. If you find it useful, and we meet someday,
  you can buy me a beer.

  This all-inclusive sketch will show off a series of graphics
  functions, like drawing lines, circles, squares, and text. Then
  it'll go into serial monitor echo mode, where you can type
  text into the serial monitor, and it'll be displayed on the
  LCD.

  This stuff could all be put into a library, but we wanted to
  leave it all in one sketch to keep it as transparent as possible.

  Hardware: (Note most of these pins can be swapped)
    Graphic LCD Pin ---------- Arduino Pin
       1-VCC       ----------------  5V
       2-GND       ----------------  GND
       3-SCE       ----------------  7
       4-RST       ----------------  6
       5-D/C       ----------------  5
       6-DN(MOSI)  ----------------  11
       7-SCLK      ----------------  13
       8-LED       - 330 Ohm res --  9
   The SCLK, DN(MOSI), must remain where they are, but the other 
   pins can be swapped. The LED pin should remain a PWM-capable
   pin. Don't forget to stick a current-limiting resistor in line
   between the LCD's LED pin and Arduino pin 9!
*/
#include "Text-Based_RPG.h"

void setup()
{
  digitalWrite(Reset_pin, HIGH);
  pinMode(JoySW_pin, INPUT);
  pinMode(Button_pin, INPUT_PULLUP);
  pinMode(Buzz_pin, OUTPUT);
  pinMode(RGBRed_pin, OUTPUT);
  pinMode(RGBGreen_pin, OUTPUT);
  pinMode(RGBBlue_pin, OUTPUT);
  pinMode(Reset_pin, OUTPUT);
  digitalWrite(JoySW_pin, HIGH);
  Serial.begin(9600);
  randomSeed(analogRead(A4));
  Serial.print(analogRead(A4));

  lcdBegin();      // This will setup our pins, and initialize the LCD
  updateDisplay(); // with displayMap untouched, SFE logo
  setContrast(40); // Good values range from 40-60
  delay(2000);

  lcdTest(); // Runs a 30-second demo of graphics functions
  delay(2000);
  initDefaultMenus();
  select(0);
}

// Loop turns the display into a local serial monitor echo.
// Type to the Arduino from the serial monitor, and it'll echo
// what you type on the display. Type ~ to clear the display.
void loop()
{
  static int cursorX = 0;
  static int cursorY = 0;
  onUpdate();
  if (buttonPressed == -1)
  {
    char input = digitalRead(Button_pin);
    if (!input)
    {
      onButtonPress(0);
    }
  }
  else if (buttonPressed == 0)
  {
    char input = digitalRead(Button_pin);
    if (input)
    {
      buttonPressed = -1;
    }
  }
  if (millis() >= nextScrollTimeStamp)
  {
    int joyX_value = analogRead(JoyX_pin);
    int joyY_value = analogRead(JoyY_pin);
    if (joyY_value > JoyHighThrehold)
    {
      onJoystickScroll(DIRECTION_DOWN);
    }
    else if (joyY_value < JoyLowThrehold)
    {
      onJoystickScroll(DIRECTION_UP);
    }
    if (joyX_value > JoyHighThrehold)
    {
      onJoystickScroll(DIRECTION_RIGHT);
    }
    else if (joyX_value < JoyLowThrehold)
    {
      onJoystickScroll(DIRECTION_LEFT);
    }
  }

  if (Serial.available())
  {
    char c = Serial.read();

    switch (c)
    {
    case '\n': // New line
      cursorY += 8;
      break;
    case '\r': // Return feed
      cursorX = 0;
      break;
    case '~': // Use ~ to clear the screen.
      clearDisplay(WHITE);
      updateDisplay();
      cursorX = 0; // reset the cursor
      cursorY = 0;
      break;
    case 'l':
      cheatLockVision = !cheatLockVision;
      break;
    case 'u':
      player.replenish(200, 100);
      break;
    case 'd':
      player.replenish(-200, -100);
      break;
    default:
      setChar(c, cursorX, cursorY, BLACK);
      updateDisplay();
      cursorX += 6; // Increment cursor
      break;
    }
    // Manage cursor
    if (cursorX >= (LCD_WIDTH - 4))
    {               // If the next char will be off screen...
      cursorX = 0;  // ... reset x to 0...
      cursorY += 8; // ...and increment to next line.
      if (cursorY >= (LCD_HEIGHT - 7))
      {              // If the next line takes us off screen...
        cursorY = 0; // ...go back to the top.
      }
    }
  }
  if (menus[menuIndex]->UsesLEDs())
  {
    float healthRatio = ((float)player.health) / player.healthMax;
    float manaRatio = ((float)player.mana) / player.manaMax;
    float greenRatio = max(healthRatio - 0.25, 0) * 4 / 3;
    float redRatio = 1 - 2 * abs(healthRatio - 0.5);
    setRGBColors(255 * redRatio, 255 * greenRatio, 255 * manaRatio);
  }
  else
    setRGBColors(0, 0, 0);
}

void lcdTest()
{
  clearDisplay(WHITE); // Begin by clearing the display
  char title[] = "Text-Based RPG";
  const int chrSize = 84 / CHR_PER_ROW;
  setStr("Text-Based RPG", 0, LCD_HEIGHT / 2 - 16, WHITE);
  setStr("by Alexandru", chrSize, LCD_HEIGHT / 2, BLACK);
  setStr("Gabriel Rait", chrSize, LCD_HEIGHT / 2 + 8, BLACK);
  setStr("'", 12 * chrSize, LCD_HEIGHT / 2 + 16, BLACK);
  updateDisplay();
  delay(750);
}